unit UFormMain;

interface

uses
  XMLConf, Classes, SysUtils, FileUtil, Forms, Controls, Graphics,
  Dialogs, ExtCtrls, ComCtrls, Menus, ActnList, UEngine, UPlatform, Math,
  DateUtils, GraphType, UPersistentForm, UApplicationInfo, UTranslator,
  LCLType, URegistry, UAboutDialog, DOM;

type

  { TFormMain }

  TFormMain = class(TForm)
    AAbout: TAction;
    AboutDialog1: TAboutDialog;
    AShowRawImageDesc: TAction;
    AShowMap: TAction;
    ANewGame: TAction;
    AExit: TAction;
    AFullScreen: TAction;
    ActionList1: TActionList;
    Image1: TImage;
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem9: TMenuItem;
    StatusBar1: TStatusBar;
    TimerDraw: TTimer;
    TimerEngineTick: TTimer;
    XMLConfig1: TXMLConfig;
    procedure AAboutExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AFullScreenExecute(Sender: TObject);
    procedure ANewGameExecute(Sender: TObject);
    procedure AShowMapExecute(Sender: TObject);
    procedure AShowRawImageDescExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormDeactivate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure Image1Resize(Sender: TObject);
    procedure TimerDrawTimer(Sender: TObject);
    procedure TimerEngineTickTimer(Sender: TObject);
  private
    StartTime: TDateTime;
    Drawing: Boolean;
    FullScreenEnabled: Boolean;
    procedure LoadConfig;
    procedure SaveConfig;
  public
    PersistentForm: TPersistentForm;
    Engine: TEngine;
    procedure EraseBackground(DC: HDC); override;
  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UCore, UFormMap, UFormNewGame;

resourcestring
  SRound = '%0:s of %1:s';

{ TFormMain }

procedure TFormMain.TimerDrawTimer(Sender: TObject);
begin
  Application.ProcessMessages;
  if not Drawing then
  try
    Drawing := True;
    StartTime := NowPrecise;
    //Engine.Draw;
    if Assigned(Engine) then begin
      Engine.Lock.Acquire;
      try
        //StatusBar1.Panels[1].Text := IntToStr(TPlayer(Engine.Players[0]).Position.X) + ', ' +
        //  IntToStr(TPlayer(Engine.Players[0]).Position.Y) + ' ' +
        //  IntToStr(TPlayer(Engine.Players[0]).Direction);
        StatusBar1.Panels[2].Text := FloatToStr(RoundTo(Engine.DrawDuration / OneMillisecond, -2));
        StatusBar1.Panels[3].Text := Format(SRound, [IntToStr(Engine.CurrentRound),
          IntToStr(Engine.MaxRound)]);
      finally
        Engine.Lock.Release;
      end;
    end;
  finally
    Drawing := False;
  end;
  if Engine.ThreadErrorMessage <> '' then begin
    ShowMessage(Engine.ThreadErrorMessage);
    Engine.ThreadErrorMessage := '';
  end;
end;

procedure TFormMain.TimerEngineTickTimer(Sender: TObject);
begin
  //Engine.Tick;
end;

procedure TFormMain.LoadConfig;
begin
  if not FileExists(XMLConfig1.Filename) then Exit;

  with XMLConfig1 do begin
    FullScreenEnabled := GetValue(DOMString('LanguageCode'), FullScreenEnabled);
  end;

  if Assigned(Engine) then Engine.LoadConfig(XMLConfig1, '');
end;

procedure TFormMain.SaveConfig;
begin
  with XMLConfig1 do begin
    SetValue(DOMString('LanguageCode'), FullScreenEnabled);
  end;
  if Assigned(Engine) then Engine.SaveConfig(XMLConfig1, '');
  ForceDirectories(ExtractFileDir(XMLConfig1.Filename));
  XMLConfig1.Flush;
end;

procedure TFormMain.EraseBackground(DC: HDC);
begin
  // Do nothing
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  XMLConfig1.Filename := GetAppConfigDir(False) + 'Config.xml';

  PersistentForm := TPersistentForm.Create(nil);
  PersistentForm.RegistryContext := TRegistryContext.Create(Core.ApplicationInfo.RegistryRoot,
    Core.ApplicationInfo.RegistryKey);

  Application.OnDeactivate := FormDeactivate;
  Randomize;
  DoubleBuffered := False;
  Engine := TEngine.Create(nil);
  Engine.Bitmap := Image1.Picture.Bitmap;
  Core.Translator1.Language := Core.Translator1.Languages.SearchByCode('cs');
  LoadConfig;
  Engine.NewGame;
  Engine.Active := True;
  Image1Resize(Self);
  Image1.ControlStyle := Image1.ControlStyle + [csOpaque];
end;

procedure TFormMain.FormDeactivate(Sender: TObject);
begin
  if Assigned(Engine) then Engine.KeyBoard.ClearAll;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  FreeAndNil(PersistentForm);
  FreeAndNil(Engine);
end;

procedure TFormMain.AFullScreenExecute(Sender: TObject);
begin
  FullScreenEnabled := not FullScreenEnabled;
  PersistentForm.SetFullScreen(FullScreenEnabled);
end;

procedure TFormMain.ANewGameExecute(Sender: TObject);
begin
  FormNewGame := TFormNewGame.Create(nil);
  try
    FormNewGame.LoadData(Engine);
    if FormNewGame.ShowModal = mrOk then begin
      FormNewGame.SaveData(Engine);
      Engine.NewGame;
    end;
  finally
    FreeAndNil(FormNewGame);
  end;
end;

procedure TFormMain.AShowMapExecute(Sender: TObject);
begin
  if not Assigned(FormMap) then FormMap := TFormMap.Create(Self);
  FormMap.Show;
end;

procedure TFormMain.AShowRawImageDescExecute(Sender: TObject);
begin
  ShowMessage(Image1.Picture.Bitmap.RawImage.Description.AsString);
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  PersistentForm.Save(Self);
  SaveConfig;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AAboutExecute(Sender: TObject);
begin
  AboutDialog1.Show;
end;

procedure TFormMain.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Engine.KeyDown(Key);
  StatusBar1.Panels[0].Text := IntToStr(Key);
end;

procedure TFormMain.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
  );
begin
  Engine.KeyUp(Key);
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  PersistentForm.RegistryContext := Core.ApplicationInfo.GetRegistryContext;
  PersistentForm.Load(Self, False, True);
  FullScreenEnabled := PersistentForm.FormFullScreen;
  {$IFDEF DEBUG}
  StatusBar1.Visible := True;
  {$ENDIF}
end;

procedure TFormMain.Image1Resize(Sender: TObject);
begin
  Engine.Lock.Acquire;
  try
    Image1.Picture.Bitmap.SetSize(Image1.Width, Image1.Height);
    Engine.ResizePlayerFrames;
  finally
    Engine.Lock.Release;
  end;
end;

end.

